local _WIDTH = love.window.getWidth()
local _HEIGHT = love.graphics.getHeight()

local hasStarted = false

local player = {img = nil,  x = _WIDTH / 2, y = _HEIGHT / 2 + (300), hspeed = 150, vspeed = 90, w = 110, h = 86}
local bg = {img = nil}

local lives = 3
local isAlive = true

local score = 0

local canShoot = true
local canShootTimerMax = 0.35
local canShootTimer = canShootTimerMax

local bulletImg = nil
local bullets = {}

local canSpawnEnemy = false
local canSpawnEnemyTimerMax = 1.7
local canSpawnEnemyTimer = canSpawnEnemyTimerMax

local enemyImg = nil
local enemies = {}

local shoots = 0

function CheckCollision(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 < x2+w2 and
         x2 < x1+w1 and
         y1 < y2+h2 and
         y2 < y1+h1
end

function love.load()
	--Load assets
	bg.img = love.graphics.newImage('assets/bg.png')
	player.img = love.graphics.newImage('assets/spr_player.png')
	bulletImg = love.graphics.newImage('assets/spr_bullet.png')
	enemyImg = love.graphics.newImage('assets/spr_enemy.png')

	snd_explosion = love.audio.newSource('assets/explosion.wav','static')
	snd_shoot = love.audio.newSource('assets/shoot.wav','static')
	snd_bulletcoll = love.audio.newSource('assets/bulletcoll.wav','static')
	snd_pexplosion = love.audio.newSource('assets/pexplosion.wav','static')

	font = love.graphics.newFont('assets/font.ttf',28)
end

function love.update( dt )
	love.window.setTitle('Scrolling Shooter FPS: ' .. tostring(love.timer.getFPS()) )
	if love.keyboard.isDown('escape') then love.event.push('quit') end

	if isAlive then
		if hasStarted then
					--PLAYER INPUT
			if love.keyboard.isDown('left','a') then 
				if(player.x > 0) then
					player.x = player.x - (player.hspeed*dt) 
				end
			end

			if love.keyboard.isDown('right','d') then 
				if(player.x < _WIDTH) then 
					player.x = player.x + (player.hspeed*dt)
				end 
			end

			if love.keyboard.isDown('up','w') then 
				if(player.y > 0) then
					player.y = player.y - (player.vspeed * dt) 
				end
			end

			if love.keyboard.isDown('down','s') then 
				if(player.y < _HEIGHT) then
					player.y = player.y + (player.vspeed * dt) 
				end
			end

			if love.keyboard.isDown(' ') then
				
				if(canShoot == true) then
					newBullet = {x = player.x - 5 - 30, y = player.y - 20, img = bulletImg}
					newBullet0 = {x = player.x - 5 + 30, y = player.y - 20, img = bulletImg}

					table.insert(bullets, newBullet)
					table.insert(bullets, newBullet0)

					canShoot = false
					canShootTimer = canShootTimerMax

					snd_shoot:play()

					shoots = shoots + 1
					print("times you shooted: " .. shoots)
				end
			end

			--Enemy spawner
			if(canSpawnEnemy == true) then
				canSpawnEnemy = false
				canSpawnEnemyTimer = canSpawnEnemyTimerMax
				
				enemy = {img = enemyImg,
						x = love.math.random(enemyImg:getWidth(), _WIDTH - enemyImg:getWidth() ),
						y = -enemyImg:getHeight(),
						lives = 4,
						randomspd = love.math.random(100)}
				table.insert(enemies, enemy)
			end
			--SHOOT TIMER
			canShootTimer = canShootTimer - (1 * dt)
			if(canShootTimer < 0) then
				canShoot = true
			end

			--SPAWN ENEMY TIMER
			canSpawnEnemyTimer = canSpawnEnemyTimer - (1 * dt)
			if(canSpawnEnemyTimer < 0) then
				canSpawnEnemy = true
			end
		else
			if(love.keyboard.isDown(' '))then
				hasStarted = true
			end
		end
	elseif isAlive == false then
		if love.keyboard.isDown('r') then
			lives = 3
			score = 0
			player.x = _WIDTH / 2
			player.y = _HEIGHT / 2 + (300)
		end
	end
	--Entities update
	for i, bullet in ipairs(bullets) do
		bullet.y = bullet.y - (350 * dt)

	  	if bullet.y < 0 - bullet.img:getHeight() then
			table.remove(bullets, i)
		end
	end
	--Check the collisions
	for i,enemy in ipairs(enemies) do
		for j,bullet in ipairs(bullets) do
			--collision with the bullet - enemy
			if CheckCollision(enemy.x,enemy.y,enemy.img:getWidth(),enemy.img:getHeight(),bullet.x,bullet.y,bullet.img:getWidth(),bullet.img:getHeight()) then
				table.remove(bullets,j)
				snd_bulletcoll:play()
				if(enemy.lives > 0) then
					enemy.lives = enemy.lives - 1
				elseif enemy.lives <= 0 then
					table.remove(enemies,i)
					snd_explosion:play()
					score = score + 1
					print(score)
				end
			end
		end
		enemy.y = enemy.y + ((250 + enemy.randomspd) * dt)
		
		if CheckCollision(enemy.x,enemy.y,enemy.img:getWidth(),enemy.img:getHeight(), player.x,player.y,player.img:getWidth(),player.img:getHeight()) and isAlive then
			table.remove(enemies,i)
			lives = lives - 1
			print("lives " .. lives)
			snd_pexplosion:play()
		elseif enemy.y > _HEIGHT then
			table.remove(enemies,i) 
		end
	end
	
	--Check if lives is greater or equal than 0, else, isAlive will be false
	if (lives <= 0) then
		isAlive = false
	else
		isAlive = true
	end

end

function love.draw()
	love.graphics.setFont(font)
	love.graphics.draw(bg.img, 0, 0);
	
	love.graphics.setColor(0,0,0,255)
	if hasStarted then
		love.graphics.print("Score " .. tostring(score), 12, 0 + (24)) -- print score and 
		love.graphics.print("Lives " .. tostring(lives), 12, 0 + (64)) -- lives on the screen
	else 
		love.graphics.printf('Press SPACEBAR to start', _WIDTH / 2 - 200, _HEIGHT / 2, 400,'center')
	end
	love.graphics.setColor(255,255,255,255)

	for i,bullet in ipairs(bullets) do
		love.graphics.draw(bullet.img,bullet.x,bullet.y)
	end

	for i, enemy in ipairs(enemies) do
		love.graphics.draw(enemy.img,enemy.x,enemy.y)	
	end

	if isAlive then
		love.graphics.draw(player.img, player.x - (player.w / 2), player.y - (player.h / 2) )	
	else 
		love.graphics.setColor(0,0,0,255)
		love.graphics.printf("GAME OVER ", _WIDTH / 2, _HEIGHT / 2,1,'center')
		love.graphics.printf("PRESS R TO RESTART", _WIDTH / 2 - (300 / 2), _HEIGHT / 2 + 128, 300,'center')
		love.graphics.setColor(255,255,255,255)
	end
end